
#ifndef OPENSSH_WIN32_H
#define OPENSSH_WIN32_H

#ifndef OPENSSH_DISABLE_IPV6
   /* Set to 0 to enable IPv6 (NT/Win2000) by default, and 
    * 1 for IPv4 only. 0 will giv backwards system
	* compatibility with win9x, while the IPv6 library
	* require Windows NT/Windows 2000.
    */
#	define OPENSSH_DISABLE_IPV6 1
#endif

#define SSH_WINVERSION "WIN32_0.1"

#include <winsock2.h>
#if !OPENSSH_DISABLE_IPV6
#include <ws2ip6.h>
#endif
#include <windows.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <stddef.h>
#include <malloc.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <io.h>
#include <direct.h>
#include <process.h>

typedef HANDLE uid_t;
typedef short gid_t;
typedef short mode_t;
typedef DWORD pid_t;
typedef int ssize_t;
typedef __int8 int8_t;
typedef __int16 int16_t;
typedef __int32 int32_t;
typedef unsigned __int8 u_int8_t;
typedef unsigned __int16 u_int16_t;
typedef unsigned __int32 u_int32_t;
typedef unsigned int speed_t;
typedef u_int16_t port_t;
#define sockaddr_storage sockaddr
#define ss_family sa_family 
struct winsize {
	unsigned short	ws_row;		/* rows, in characters */
	unsigned short	ws_col;		/* columns, in characters */
	unsigned short	ws_xpixel;	/* horizontal size, pixels */
	unsigned short	ws_ypixel;	/* vertical size, pixels */
};

#define __P(a) a
#define __attribute__(a)
#define arc4random_stir()
#define log ssh_log

#define strlcpy(a,b,c) (strlen(strncpy(a,b,c)))
#define strlcat(a,b,c) (strlen(strncat(a,b,c)))
#define snprintf _snprintf
#define vsnprintf _vsnprintf
#define popen _popen
#define pclose _pclose
#define pipe(a) _pipe(a, 1024, _O_NOINHERIT | _O_TEXT)
#define strcasecmp(a,b) stricmp(a,b)
#define __progname _pgmptr
#define MAP_HOST_CONFIG_FILE get_host_config_file()
#define sleep(a) Sleep(a * 1000)
#define mkdir(a,b) _mkdir(a)
#define strtok_r(a,b,c) strtok(a,b) /* strtok is thread safe in Win32 */

#ifndef MAXPATHLEN
#	define MAXPATHLEN _MAX_PATH
#endif
#define MAXHOSTNAMELEN 256 /* jgaa oct 9th 2000: I assume RFC 1035 label name (domain name len) */
#define SHUT_RDWR (SD_RECEIVE | SD_SEND)
#define SHUT_RD (SD_RECEIVE)
#define SHUT_WR (SD_SEND)
#define STDIN_FILENO fileno(stdin)
#define STDOUT_FILENO fileno(stdout)
#define STDERR_FILENO fileno(stderr)
#define SSH_IS_ROOT(uid) (1) /*win32port_is_administrator()*/

#define F_SETFL 1
#define F_GETFL 2
#define O_NONBLOCK 1

#define TIOCGWINSZ 1
#define IN_LOOPBACKNET (INADDR_LOOPBACK >> 24)

u_int32_t arc4random(void);
const char* win32port_get_ssh_data_path();
int win32port_unprotected_file(const char *filename);
int win32port_get_registry_value(HKEY root, 
								 const char *key, 
								 const char *value, 
								 char *return_buf, 
								 DWORD *return_buf_len);
char *strsep(char **stringp, const char *delim);
int win32port_set_non_blocking(int sock);
#define __b64_ntop b64_ntop
int b64_ntop(u_char const *src, size_t srclength, char *target, size_t targsize);
#define __b64_pton b64_pton
int b64_pton(src, target, targsize);
const char *win32port_getusername(int force_refresh);
int getopt(int argc, char * const *argv, const char *optstring);
const char* win32port_get_ssh_data_path(const char *filename);
const char* get_host_config_file(void);
const char *win32port_get_canonical_name(void);
int daemon(int nochdir, int noclose);
BOOL win32port_is_administrator(VOID);
int gettimeofday(struct timeval *tv, struct timezone *tz);
int ioctl(int d, unsigned long request, ...);
int rresvport_af(int *port, int af);
char *win32port_mkdtemp(char *template);
int win32port_fcntl(int fd, int cmd, ...);
uid_t win32port_getuid(void);
uid_t win32port_geteuid(void);
int win32port_setuid(uid_t uid);
int win32port_seteuid(uid_t euid);
ssize_t win32port_read(int d, char *buf, size_t nbytes);
ssize_t win32port_write(int d, const char *buf, size_t nbytes);

#define mkdtemp win32port_mkdtemp
#define fcntl win32port_fcntl
#define getuid win32port_getuid
#define geteuid win32port_geteuid
#define setuid win32port_setuid
#define seteuid win32port_seteuid
#define read win32port_read
#define write win32port_write

#if OPENSSH_DISABLE_IPV6
#	include "win32port_netdb.h"
#endif

#include "win32port_AF_UNIX.h"
#include "win32port_termios.h"
#include "win32port_pwd.h"
#include "win32port_select.h"

#ifndef _PATH_RSH
#define _PATH_RSH "rsh.exe"
#endif


#endif /* OPENSSH_WIN32_H */

