
/* jgaa oct 12th 2000: 
 * This Implementation is ipv4 only, and intended to make
 * it possible to use the new name-resolve API on the Win32
 * platform, without installing IPv6, and on machines 
 * running Win9x/ME.
 *
 * IPv6 is available for Microsoft Windows NT and 
 * Windows2000 from here: 
 *     http://www.research.microsoft.com/msripv6/
 */

#ifndef WIN32PORT_NETDB_H
#define WIN32PORT_NETDB_H

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned int socklen_t;

struct addrinfo {
    int ai_flags;              /* AI_PASSIVE, AI_CANONNAME, AI_NUMERICHOST. */
    int ai_family;             /* PF_xxx. */
    int ai_socktype;           /* SOCK_xxx. */
    int ai_protocol;           /* 0 or IPPROTO_xxx for IPv4 and IPv6. */
    size_t ai_addrlen;         /* Length of ai_addr. */
    char *ai_canonname;        /* Canonical name for nodename. */
    struct sockaddr *ai_addr;  /* Binary address. */
    struct addrinfo *ai_next;  /* Next structure in linked list. */

	/* non-portable extensions */
	struct sockaddr ai_buf_sockaddr; /* buffer pointet to by ai_addr */
};


/* Error codes from getaddrinfo() */

#define EAI_ADDRFAMILY  1   /* Address family for nodename not supported. */
#define EAI_AGAIN       2   /* Temporary failure in name resolution. */
#define EAI_BADFLAGS    3   /* Invalid value for ai_flags */
#define EAI_FAIL        4   /* Non-recoverable failure in name resolution. */
#define EAI_FAMILY      5   /* Address family ai_family not supported. */
#define EAI_MEMORY      6   /* Memory allocation failure. */
#define EAI_NODATA      7   /* No address associated with nodename. */
#define EAI_NONAME      8   /* Nodename nor servname provided, or not known. */
#define EAI_SERVICE     9   /* Servname not supported for ai_socktype. */
#define EAI_SOCKTYPE    10  /* Socket type ai_socktype not supported. */
#define EAI_SYSTEM      11  /* System error returned in errno. */

/* Flags used in "hints" argument to getaddrinfo() */
#define AI_PASSIVE     0x1  /* Socket address will be used in bind() call. */
#define AI_CANONNAME   0x2  /* Return canonical name in first ai_canonname.*/
#define AI_NUMERICHOST 0x4  /* Nodename must be a numeric address string. */

#define NI_MAXHOST  1025  /* Max size of a fully-qualified domain name. */
#define NI_MAXSERV    32  /* Max size of a service name. */

/* Flags for getnameinfo(). */

#define NI_NOFQDN       0x01  /* Only return nodename portion for local hosts.  */
#define NI_NUMERICHOST  0x02  /* Return numeric form of the host's address. */
#define NI_NAMEREQD     0x04  /* Error if the host's name not in DNS. */
#define NI_NUMERICSERV  0x08  /* Return numeric form of the service (port #). */
#define NI_DGRAM        0x10  /* Service is a datagram service. */

#define WIN32PORT_NETDB_RETERR(err) return err;

#define getnameinfo win32port_getnameinfo
#define getaddrinfo win32port_getaddrinfo
#define freeaddrinfo win32port_freeaddrinfo
#define gai_strerror win32port_gai_strerror

int win32port_getnameinfo(const struct sockaddr *sa, 
						  socklen_t salen, 
						  char *host, size_t hostlen, 
						  char *serv, size_t servlen, 
						  int flags);

int win32port_getaddrinfo(const char *nodename, 
						  const char *servname,
						  const struct addrinfo *hints, 
						  struct addrinfo **res);

void win32port_freeaddrinfo(struct addrinfo *ai);

const char *win32port_gai_strerror(int ecode);

#ifdef __cplusplus
}
#endif
#endif WIN32PORT_NETDB_H
