
/* jgaa oct 11th 2000: 
* Under Windows, we don't have AF_UNIX sockets. 
* We'll therefore map AF_UNIX to AF_INET and use
* the sun_path as a numerical representation of
* a local port number.
*/

#include "includes.h"

#undef connect
#undef bind

int win32port_connect(int s, 
					  const struct sockaddr *name, 
					  socklen_t namelen) {	

	if (namelen == sizeof(struct sockaddr_un)) {
		if (name->sa_family == AF_UNIX) {
			struct sockaddr tmp_addr;
			struct sockaddr_un *pu_name = (struct sockaddr_un *)name;
			struct sockaddr_in* pin_tmp = (struct sockaddr_in *)&tmp_addr;
			struct sockaddr_in* pin_name = (struct sockaddr_in *)name;
			const char *p;
			
			for(p = pu_name->sun_path; *p; p++) {
				if (!isdigit(*p)) 
					goto einval; /* Path must be a port number */
			}
			
			memset(&tmp_addr, 0, sizeof(tmp_addr));
			pin_tmp->sin_family = AF_INET;
			pin_tmp->sin_port =  htons((port_t)(atoi(pu_name->sun_path)));
			pin_tmp->sin_addr.s_addr = INADDR_LOOPBACK;
			
			if (pin_tmp->sin_port == 0)
				goto einval;
			
			pin_name->sin_addr.s_addr = pin_tmp->sin_addr.s_addr;
			pin_name->sin_port = pin_tmp->sin_port;
			
			return connect(s, &tmp_addr, sizeof(tmp_addr));
		} 
		return connect(s, name, sizeof(struct sockaddr));
	}
	
	return connect(s, name, namelen);
	
	
einval:
	WSASetLastError (WSAEINVAL);
	SetLastError(WSAEINVAL);
	errno = EINVAL;
	return -1;
}

int win32port_bind(int s, const struct sockaddr *name, socklen_t namelen) {
	
	if (namelen == sizeof(struct sockaddr_un)) {
		if (name->sa_family == AF_UNIX) {
			struct sockaddr tmp_addr;
			struct sockaddr_un *pu_name = (struct sockaddr_un *)name;
			struct sockaddr_in* pin_tmp = (struct sockaddr_in *)&tmp_addr;
			struct sockaddr_in* pin_name = (struct sockaddr_in *)name;
			const char *p;
			
			for(p = pu_name->sun_path; *p; p++) {
				if (!isdigit(*p)) 
					goto einval; /* Path must be a port number */
			}
			
			memset(&tmp_addr, 0, sizeof(tmp_addr));
			pin_tmp->sin_family = AF_INET;
			pin_tmp->sin_port =  htons((port_t)(atoi(pu_name->sun_path)));
			pin_tmp->sin_addr.s_addr = INADDR_LOOPBACK;
			
			if (pin_tmp->sin_port == 0)
				goto einval;
			
			pin_name->sin_addr.s_addr = pin_tmp->sin_addr.s_addr;
			pin_name->sin_port = pin_tmp->sin_port;
			
			return bind(s, &tmp_addr, sizeof(tmp_addr));
		} 
		return bind(s, name, sizeof(struct sockaddr));
	}
	
	return bind(s, name, namelen);
	
	
einval:
	WSASetLastError (WSAEINVAL);
	SetLastError(WSAEINVAL);
	errno = EINVAL;
	return -1;
}
