
#include "includes.h"
#include <assert.h>

int win32port_fcntl_nonblocking = 0;

int win32port_fcntl(int fd, int cmd, ...) {
	va_list marker;
	int rval = -1;
	
	va_start(marker, cmd);
	
	switch(cmd) {
		
	case F_SETFL:
		{
			int what = va_arg(marker, int);
			u_long opt = (what & O_NONBLOCK) ? 1 : 0;

			rval = ioctlsocket(fd, FIONBIO, &opt);
			if (fd == 0)
				win32port_fcntl_nonblocking = (int)opt;
			if (rval == -1) {
				if (WSAGetLastError() == WSAENOTSOCK)
					rval = 0;
			}
		}
		break;

	case F_GETFL:
		if (fd == 0) {
			int* what = va_arg(marker, int *);
			*what = win32port_fcntl_nonblocking ? O_NONBLOCK : 0;
		} else {
			/* Unsupported command */
			;
		}
		rval = 0; /* We cannot detect O_NONBLOCK state */
		break;
		
	default:
		assert(0); /* Unsupported command */
	}
	
	va_end(marker);
	
	if ((rval == -1) && (errno == 0))
		errno = EINVAL;
	
	return rval;
}
