
#include "includes.h"

#ifdef WIN32PORT_NETDB_H

#include <assert.h>

#define MAX_ADDR 128
#define NUM_SERVICES 2
const static char *service_name[NUM_SERVICES] = {"tcp", "udp"};
const static int service_prot[NUM_SERVICES] = {SOCK_STREAM, SOCK_DGRAM};
#define CURR_IN(a) ((struct sockaddr_in *)pnew_ap->ai_addr)->a

int win32port_getaddrinfo(const char *nodename, 
						  const char *servname,
						  const struct addrinfo *hints, 
						  struct addrinfo **res) 
{
	struct sockaddr sa;
	struct sockaddr_in *psa = (struct sockaddr_in *)&sa;
	struct in_addr addr[MAX_ADDR];
	port_t service_port[NUM_SERVICES] = {0,0};
	int i, service_cnt = 0, addr_cnt = 0, curr_service, curr_addr;
	struct hostent* phostent = NULL;
	struct addrinfo* pnew_ap, *prev_ap = NULL; 
	
	memset(&sa, 0, sizeof(sa));
	
	if (!res  || (!nodename && !servname)) {
		WIN32PORT_NETDB_RETERR(EAI_FAIL)
	}
	
	*res = NULL;
	
	if (servname) {
		struct servent *pservent = NULL;
		int serv_port = 0;
		const char *p;

		/* See if sercvice is a portnumber */
		for(p = servname; ; ++p) {
			if (!isdigit(*p))
				break;
		}
		if (!*p)
			serv_port = atoi(servname);
		
		for(i = 0; i < NUM_SERVICES; i++) {
			if ((hints != NULL)
				&& (hints->ai_protocol != 0)
				&& (hints->ai_protocol != service_prot[i]))
				continue;
			if (serv_port) {
				service_port[i] = serv_port;
				service_cnt++;
			}
			else if (pservent = getservbyname(servname, service_name[i])) {
				service_port[i] = pservent->s_port;
				service_cnt++;
			}
		}
		
		if (service_cnt == 0) {
			WIN32PORT_NETDB_RETERR(EAI_SERVICE)
		}
	} else {
		/* Just add dummy port numbers */
		for(i = 0; i < NUM_SERVICES; i++) {
			if ((hints != NULL)
				&& (hints->ai_protocol != 0)
				&& (hints->ai_protocol != service_prot[i]))
				continue;
			
			service_port[i] = 0; /* dummy */
			service_cnt++;
		}
	}
	
	if (nodename) {
		
		/* Try IP resolution */
		if ((addr[0].s_addr = inet_addr(nodename)) == INADDR_NONE) {
			/* Try DNS */
			if ((hints != NULL) && (hints->ai_flags & AI_NUMERICHOST)) {
				WIN32PORT_NETDB_RETERR(EAI_NONAME)
			}
			
			if ((phostent = gethostbyname(nodename)) == NULL) {
				WIN32PORT_NETDB_RETERR(EAI_NONAME)
			}
			
			{
				char **p;
				for(p = phostent->h_addr_list; *p != 0; p++) {
					memcpy(&addr[addr_cnt++].s_addr, *p, 
						sizeof (addr[0].s_addr));
				}
			}
		}
		else {
			/* IP name resolution  succeeded */
			addr_cnt = 1;
		}
		
		if (addr_cnt == 0) {
			WIN32PORT_NETDB_RETERR(EAI_NONAME)
		}
	} else {
		addr_cnt = 1;
		if ((hints != NULL)
			&& (hints->ai_flags & AI_PASSIVE)) 
			addr[0].s_addr = htonl(INADDR_ANY);
		else
			addr[0].s_addr = htonl(INADDR_LOOPBACK); /* Is this common practice ?? */
	}
	
	for(curr_addr = 0; curr_addr < addr_cnt; curr_addr++) {
		for(curr_service = 0; curr_service < service_cnt; curr_service++) {
			pnew_ap = (struct addrinfo *)malloc(sizeof(struct addrinfo));
			if (pnew_ap == NULL) 
out_of_memory:
			{
				freeaddrinfo(*res);
				WIN32PORT_NETDB_RETERR(EAI_MEMORY)
			}
			memset(pnew_ap, 0, sizeof(struct addrinfo));

			/* Link */
			if (!*res)
				*res = pnew_ap;
			else
				 prev_ap->ai_next = pnew_ap;;
			prev_ap = pnew_ap;

			/* Assign values */
			pnew_ap->ai_addr = &pnew_ap->ai_buf_sockaddr;
			CURR_IN(sin_family) = AF_INET; /* All we support for now ... */
			CURR_IN(sin_port) = htons(service_port[curr_service]);
			CURR_IN(sin_addr).s_addr = addr[curr_addr].s_addr;
			
			pnew_ap->ai_flags = 0;
			pnew_ap->ai_family = CURR_IN(sin_family);
			pnew_ap->ai_addrlen = sizeof(struct sockaddr);
			pnew_ap->ai_socktype = service_prot[curr_service];
			pnew_ap->ai_protocol = IPPROTO_TCP;
			if (!*res && hints && (hints->ai_flags & AI_CANONNAME)) {
				if (phostent && phostent->h_name)
					pnew_ap->ai_canonname = strdup(phostent->h_name);
				else
					pnew_ap->ai_canonname = strdup("");
				if (pnew_ap->ai_canonname == NULL) 
					goto out_of_memory;
			}
		}
	}
	
	return 0;
}

void win32port_freeaddrinfo(struct addrinfo *ai) {
	struct addrinfo *curr, *next = NULL;

	for(curr = ai; curr; curr = next) {
		next = curr->ai_next;
		if (curr->ai_canonname)
			free(curr->ai_canonname);
		free(curr);
	}
}

const char *win32port_gai_strerror(int ecode) {
	const char *perr = "Unknown EAI_* error";

	switch(ecode) {
	case EAI_ADDRFAMILY: perr = "Address family for nodename not supported."; break;
	case EAI_AGAIN:      perr = "Temporary failure in name resolution."; break;
	case EAI_BADFLAGS:	 perr = "Invalid value for ai_flags"; break;
	case EAI_FAIL:		 perr = "Non-recoverable failure in name resolution."; break;
	case EAI_FAMILY:	 perr = "Address family ai_family not supported."; break;
	case EAI_MEMORY:	 perr = "Memory allocation failure."; break;
	case EAI_NODATA:	 perr = "No address associated with nodename."; break;
	case EAI_NONAME:	 perr = "Nodename nor servname provided, or not known."; break;
	case EAI_SERVICE:	 perr = "Servname not supported for ai_socktype."; break;
	case EAI_SOCKTYPE:	 perr = "Socket type ai_socktype not supported."; break;
	case EAI_SYSTEM:	 perr = "System error returned in errno."; break;
	default:
		; /* No action */
	}
	return perr;
}

#endif /* WIN32PORT_NETDB_H */

