
#include "includes.h"

#ifdef WIN32PORT_NETDB_H

#include <assert.h>

int win32port_getnameinfo(const struct sockaddr *sa, 
						  socklen_t salen, 
						  char *host, size_t hostlen, 
						  char *serv, size_t servlen, 
						  int flags) 
{
	const struct sockaddr_in *pin = (const struct sockaddr_in *)sa;
	
	if (flags & ~(NI_NOFQDN | NI_NUMERICHOST | NI_NAMEREQD | NI_NUMERICSERV
		| NI_DGRAM)) {
		WIN32PORT_NETDB_RETERR(EAI_BADFLAGS)
	}
	
	if (flags & NI_NOFQDN) {
#if _DEBUG
		assert(0); /* Not supported */
#endif
		/* Ignored */
	}

	if (!sa || !salen 
		|| (!host && hostlen)
		|| (!serv && servlen) 
		|| ((flags & (NI_NAMEREQD | NI_NUMERICHOST)) 
		== ((NI_NAMEREQD | NI_NUMERICHOST)))) {
		WIN32PORT_NETDB_RETERR(EAI_FAIL)
	}
	
	if ((sa->sa_family != AF_INET)
		&& (sa->sa_family != AF_UNSPEC)) {
		WIN32PORT_NETDB_RETERR(EAI_ADDRFAMILY)
	}
	
	if (servlen)
	{
		if (flags & NI_NUMERICSERV)
			snprintf(serv, servlen, "%d", ntohs(pin->sin_port));
		else
		{
			const char *proto = (flags & NI_DGRAM) ? "udp" : "tcp";
			struct servent *pservent = getservbyport(ntohs(pin->sin_port), proto);
			
			serv[0] = 0;
			
			if (!pservent) {
				WIN32PORT_NETDB_RETERR(EAI_SERVICE)
			}
			
			snprintf(serv, servlen, "%s", pservent->s_name);		
		}	
	}
	
	if (hostlen) {
		host[0] = 0;
		if ((flags & NI_NUMERICHOST) == 0) {
			/* Try DNS */
			struct hostent *phostent = gethostbyaddr(
				(const char *)&sa, salen, sa->sa_family);
			
			if (phostent) {
				strncpy(host, phostent->h_name, hostlen);
			}
			else 
				goto fallback_to_ip;
		}
		else 
fallback_to_ip:
		{
			const char *ip_str = inet_ntoa(pin->sin_addr);
			
			if (flags & NI_NAMEREQD) {
				WIN32PORT_NETDB_RETERR(EAI_NODATA)
			}
			
			if (ip_str && (strlen(ip_str) < hostlen))
				strcpy(host, ip_str);
		}
		
		if (!host[0]) {
			WIN32PORT_NETDB_RETERR(EAI_FAIL)
		}
	}
	
	return 0; /* ok */
}

#endif /* WIN32PORT_NETDB_H */

