*
#include "includes.h"

/* jgaa oct 11th 2000: From the OpenBSD manual page:
 * The rresvport() and rresvport_af() functions return a descriptor to a
 * socket with an address in the privileged port space.  The iruserok() and
 * ruserok() functions are used by servers to authenticate clients request-
 * ing service with rcmd().  All four functions are present in the same file
 * and are used by the rshd(8) server (among others).  iruserok_sa() is an
 * address family independent variant of iruserok().
 */

#define MAX_RETRIES 100


int rresvport_af(int *port, int af) {
	int sock, retries;
	port_t my_port;
	struct sockaddr sa;
	struct sockaddr_in* psi = (struct sockaddr_in *)&sa;

	if (!SSH_IS_ROOT(getuid()) && !SSH_IS_EFFECTIVE_ROOT()) {
		errno = EACCES;
		return -1;
	}

	sock = socket(af, SOCK_STREAM, 0);
	memset(&sa, 0, sizeof(sa));
	psi->sin_family = af;

	if (psi->sin_addr.s_addr == INADDR_ANY) {
		/* get local soclet address */
		struct hostent *phostent = gethostbyaddr((char *)&psi->sin_addr, 
			sizeof(sa), psi->sin_family);
		if (phostent)
			memcpy(&psi->sin_addr.s_addr, phostent->h_addr_list[0],
				phostent->h_length);
	}
	
	srand(time(NULL));
	for(retries = 0; retries < MAX_RETRIES; retries++) {
		my_port = (port_t)((rand() % (IPPORT_RESERVED - 1)));
		psi->sin_port = htons(my_port);
		if (bind(sock, &sa, sizeof(sa)) == 0) {
			*port = my_port;
			return sock;
		}
	}

	closesocket(sock);
	errno = ENOENT;
	return -1;
}
