
#include "includes.h"
#include <conio.h>

#undef select
#undef FD_ISSET
#define FD_ISSET(fd, set) __WSAFDIsSet((SOCKET)(fd), (fd_set FAR *)(set))

int win32port_select(int nfds, fd_set *readfds, 
					 fd_set *writefds, fd_set *exceptfds,
					 struct timeval *timeout) 
{
	fd_set *sets[3];
	int i, index, type, type_len = sizeof(type), err, rval, num_fd = 0;
	int poll_stdin = 0, poll_cnt = 0, poll_max = 1, pending_ok = 0;
	struct timeval poll_timeout;
	fd_set my_readfds, my_writefds, my_exceptfds;

	sets[0] = readfds;
	sets[1] = writefds;
	sets[2] = exceptfds;

	FD_ZERO(&my_readfds);
	FD_ZERO(&my_writefds);
	FD_ZERO(&my_readfds);

	if (timeout)
		poll_timeout = *timeout;

	/* Remove non-socket descriptors */
	for(index = 0; index < 3; index++) {
		if (sets[index]) {
			for(i = 0; i < sets[index]->fd_count; i++) {
				if ((getsockopt(sets[index]->fd_array[i], SOL_SOCKET,
					SO_TYPE, (char *)&type, &type_len) == -1) 
					&& ((err = WSAGetLastError()) == WSAENOTSOCK)) {
					if (sets[index]->fd_array[i] == fileno(stdin))
						poll_stdin = 1;
					if ((sets[index]->fd_array[i] == fileno(stdout))
						|| (sets[index]->fd_array[i] == fileno(stderr)))
						pending_ok = 1;
					FD_CLR(sets[index]->fd_array[i], sets[index]);
				}
				else
					num_fd++;
			}
		}
	}

	if (poll_stdin) {
		DWORD origin_timeout = 0xffffffff, poll_timeout_len = 250;
		if (timeout) {
			origin_timeout = timeout->tv_usec / 10;
			origin_timeout += (timeout->tv_sec * 1000);
		}
		poll_max = origin_timeout / poll_timeout_len;
		if (poll_max < 1)
			poll_max = 1;

		poll_timeout.tv_usec = 250 * 10;
		poll_timeout.tv_sec = 0;
	}
	
	if (poll_max > 1) {
		if (readfds)
			my_readfds = *readfds;
		if (writefds)
			my_writefds = *writefds;
		if (exceptfds)
			my_exceptfds = *exceptfds;
	}
	
	for(poll_cnt = 0; poll_cnt < poll_max; poll_cnt++) {
		if (poll_max > 1) {
			if (readfds)
				*readfds = my_readfds;
			if (writefds)
				*writefds = my_writefds;
			if (exceptfds)
				*exceptfds = my_exceptfds;
		}
		
		if (num_fd == 0) { /* no socket handles */
			Sleep((poll_timeout.tv_usec / 10) + (poll_timeout.tv_sec * 1000));
		}
		else
			rval = select(nfds, readfds, writefds, exceptfds, 
				poll_stdin ? &poll_timeout : timeout);

		if (rval < 0)
			break;

		if (pending_ok)
			++rval;
		
		if (poll_stdin && _kbhit())
				++rval;

		if (rval > 0)
			break;
	}

	return rval;
}

int win32port_isset(const int fd, const fd_set *set)
{
	int type, type_len = sizeof(type), err;

	if (FD_ISSET(fd, set))
		return 1;

	if ((fd == fileno(stdin))
		&& (getsockopt(fd, SOL_SOCKET, SO_TYPE, (char *)&type, &type_len) == -1) 
		&& ((err = WSAGetLastError()) == WSAENOTSOCK)) {
		return _kbhit();
	}

	if (((fd == fileno(stdout)) || (fd == fileno(stderr)))
		&& (getsockopt(fd, SOL_SOCKET, SO_TYPE, (char *)&type, &type_len) == -1) 
		&& ((err = WSAGetLastError()) == WSAENOTSOCK)) {
		return 1;
	}

	return 0;
}

