
#include "includes.h"

static const cc_t ttydefchars[NCCS] = {
	CEOF,	CEOL,	CEOL,	CERASE, CWERASE, CKILL, CREPRINT,
		_POSIX_VDISABLE, CINTR,	CQUIT,	CSUSP,	CDSUSP,	CSTART,	CSTOP,	CLNEXT,
		CDISCARD, CMIN,	CTIME,  CSTATUS, _POSIX_VDISABLE
};


int win32port_tcgetattr(int fd, struct termios *t) {
	DWORD mode = 0;
	HANDLE h = INVALID_HANDLE_VALUE;
	size_t i;
	
	memset(t, 0, sizeof(struct termios));
	
	for(i = 0; i < NCCS; i++)
		t->c_cc[i] = ttydefchars[i];
	t->c_cflag = TTYDEF_CFLAG;
	t->c_iflag = TTYDEF_IFLAG;
	t->c_ispeed = TTYDEF_SPEED;
	t->c_lflag = TTYDEF_LFLAG & ~(ECHO | ICANON);
	t->c_oflag = TTYDEF_OFLAG;
	t->c_ospeed = TTYDEF_SPEED;
	
	
	if (fd == STDIN_FILENO)
		h = GetStdHandle(STD_INPUT_HANDLE);
	else if (fd == STDOUT_FILENO)
		h = GetStdHandle(STD_OUTPUT_HANDLE);
	else if (fd == STDERR_FILENO)
		h = GetStdHandle(STD_ERROR_HANDLE);
	
	if (h != INVALID_HANDLE_VALUE) {
		if (GetConsoleMode(h, &mode)) {
			if (mode & (ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT))
				t->c_lflag |= ICANON;
			if (mode & ENABLE_ECHO_INPUT)
				t->c_lflag |= ECHO;
		}
	}
	
	return 0;
}

int win32port_tcsetattr(int fd, int action, const struct termios *t) {
	DWORD mode = 0;
	HANDLE h = INVALID_HANDLE_VALUE;
	
	if (fd == STDIN_FILENO)
		h = GetStdHandle(STD_INPUT_HANDLE);
	else if (fd == STDOUT_FILENO)
		h = GetStdHandle(STD_OUTPUT_HANDLE);
	else if (fd == STDERR_FILENO)
		h = GetStdHandle(STD_ERROR_HANDLE);
	
	if ((h != INVALID_HANDLE_VALUE) && GetConsoleMode(h, &mode)){

		mode &= ~(ENABLE_LINE_INPUT 
			| ENABLE_PROCESSED_INPUT
			| ENABLE_ECHO_INPUT);

		if (t->c_lflag & ICANON)
			mode |= (ENABLE_LINE_INPUT | ENABLE_PROCESSED_INPUT);

		if (t->c_lflag & ECHO)
			mode |= ENABLE_ECHO_INPUT;

		SetConsoleMode(h, mode);
	}
	
	return 0;
}


